use std::fmt::{Display, Formatter};
use axum_core::response::IntoResponse;
use reqwest::StatusCode;

#[derive(Debug)]
pub enum ResolveTripNumberError {
    Reqwest(reqwest::Error),
    Serde(serde_json::Error),
    Io(std::io::Error),
    Api(String),
    ParseIntError(std::num::ParseIntError),
    Sqlx(sqlx::Error),
}

impl Display for ResolveTripNumberError {
    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
        match self {
            ResolveTripNumberError::Reqwest(e) => f.write_fmt(format_args!("Reqwest error on url={:?}: {:?} (status: {:?})", e.url(), e, e.status())),
            ResolveTripNumberError::Serde(e) => f.write_fmt(format_args!("Serde error: {:?}", e)),
            ResolveTripNumberError::Io(e) => f.write_fmt(format_args!("Io error: {:?}", e)),
            ResolveTripNumberError::Api(e) => f.write_fmt(format_args!("Api error: {:?}", e)),
            ResolveTripNumberError::ParseIntError(e) => f.write_fmt(format_args!("Parse int error: {:?}", e)),
            ResolveTripNumberError::Sqlx(e) => f.write_fmt(format_args!("Sqlx error: {:?}", e)),
        }
    }
}

impl std::error::Error for ResolveTripNumberError {}

impl From<reqwest::Error> for ResolveTripNumberError {
    fn from(err: reqwest::Error) -> Self { ResolveTripNumberError::Reqwest(err) }
}

impl From<serde_json::Error> for ResolveTripNumberError {
    fn from(err: serde_json::Error) -> Self { ResolveTripNumberError::Serde(err) }
}

impl From<std::io::Error> for ResolveTripNumberError {
    fn from(err: std::io::Error) -> Self { ResolveTripNumberError::Io(err) }
}

impl From<std::num::ParseIntError> for ResolveTripNumberError {
    fn from(err: std::num::ParseIntError) -> Self { ResolveTripNumberError::ParseIntError(err) }
}

impl From<String> for ResolveTripNumberError {
    fn from(err: String) -> Self { ResolveTripNumberError::Api(err) }
}

impl From<sqlx::Error> for ResolveTripNumberError {
    fn from(err: sqlx::Error) -> Self { ResolveTripNumberError::Sqlx(err) }
}

#[derive(Debug)]
pub enum CheckInError {
    ResolveError(ResolveTripNumberError),
    TrainNotFound,
}

impl From<ResolveTripNumberError> for CheckInError {
    fn from(err: ResolveTripNumberError) -> Self { CheckInError::ResolveError(err) }
}


impl Display for CheckInError {
    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
        match self {
            CheckInError::ResolveError(e) => f.write_str(e.to_string().as_str()),
            CheckInError::TrainNotFound => f.write_str("Train not found"),
        }
    }
}

pub type ApiResult<T> = Result<T, CheckInError>;

impl IntoResponse for CheckInError {
    fn into_response(self) -> axum_core::response::Response {
        match self {
            CheckInError::TrainNotFound => (StatusCode::OK, format!("Train not found")),
            _ => (StatusCode::INTERNAL_SERVER_ERROR, format!("Error processing checkin: {}", self)),
        }.into_response()
    }
}